@@ -51,7 +51,7 @@ def async_pool(max_thread):
         def wrapper(*args, **kwargs):
             while 1:
                 func_thread_active_count = len([i for i in threading.enumerate() if i.name == f.__name__])
-                if func_thread_active_count <= max_thread:
+                if func_thread_active_count < max_thread:
                     thr = Thread(target=f, args=args, kwargs=kwargs, name=f.__name__)
                     thr.start()
                     break
@@ -136,6 +136,7 @@ class Tool:
             self.session = requests.session()
             self.session.mount('http://', adapter)
             self.session.mount('https://', adapter)
+            self.session.verify = False
         if is_proxy_pool:
             self.update_proxy_pool()
             while True:
@@ -197,11 +198,11 @@ class Tool:
         text = text.replace('\n', '').replace('\t', '').replace('\r', '').replace(' ', '').replace('\xa0', '')
         return text
 
-    def get_html(self, url, headers={}, proxy={}, cookies={}, data={}, timeout=5, debug=False):
+    def get_html(self, url, headers={}, proxy={}, cookies={}, data={}, params={}, timeout=5, debug=False):
 
         for i in range(5):
             try:
-                res = self.session.get(url, headers=headers, timeout=timeout, proxies=proxy, cookies=cookies, data=data)
+                res = self.session.get(url, headers=headers, timeout=timeout, proxies=proxy, cookies=cookies, data=data, verify=False, params=params)
                 return res
             except Exception as e:
                 if debug:
@@ -213,7 +214,7 @@ class Tool:
 
         for i in range(5):
             try:
-                res = self.session.post(url, data=data, json=json, headers=headers, timeout=timeout, proxies=proxy, cookies=cookies)
+                res = self.session.post(url, data=data, json=json, headers=headers, timeout=timeout, proxies=proxy, cookies=cookies, verify=False)
                 return res
             except Exception as e:
                 if debug:
@@ -299,6 +300,7 @@ class Tool:
         :param data: 数据
         :return: None
         '''
+        res = 0
         if isinstance(data, dict):
             data = [data]
         if isinstance(data, list):
@@ -322,7 +324,7 @@ class Tool:
                 values = ','.join(values)
                 sql = 'insert into %s (%s) VALUES (%s)' % (table_name, key, values)
                 try:
-                    cursor.execute(sql)
+                    res = cursor.execute(sql)
                 except Exception as e:
                     if 'PRIMARY' not in str(e):
                         print(e)
@@ -330,6 +332,7 @@ class Tool:
             self.conn.commit()
             cursor.close()
             L.release()
+        return res
 
     def form_data(self, form):
         str_form = ''
@@ -401,7 +404,9 @@ class Tool:
             res = []
         finally:
             cursor.close()
+            self.conn.commit()
             L.release()
+
         return res
 
     def con_execute(self, sql):
@@ -413,9 +418,10 @@ class Tool:
             elif isinstance(sql, list):
                 for i, v in enumerate(sql):
                     cursor.execute(v)
+                    self.print_rate_of_progress(i, len(sql))
             self.conn.commit()
         except Exception as e:
-            print(e, sql)
+            print(e, "\n", sql[:500])
         finally:
             cursor.close()
             L.release()
@@ -579,18 +585,19 @@ class Tool:
         Q.put()
 
     def get_proxy(self):
-        proxy = self.con('select * from proxy order by rand() limit 1', update_cursor=True)[0]
+        proxy = self.con('select * from local.proxy where error_count < 2 order by rand() limit 1')[0]
+        id = proxy['id']
         proxy = "%s:%s" % (proxy['ip'], proxy['port'])
         proxy = {
             "http": proxy,
             "https": proxy,
         }
-        return proxy
+        return id, proxy
 
     @async_
     def update_proxy_pool(self):
         while True:
-            proxies = self.con('select * from proxy', update_cursor=True)
+            proxies = self.con('select * from proxy')
             L.acquire()
             self.proxy_pool = []
             for proxy in proxies:
@@ -677,6 +684,7 @@ class Tool:
         insert_url = """INSERT INTO {} ({}) VALUES ({}) ON DUPLICATE KEY UPDATE {}"""
         if isinstance(datas, dict):
             datas = [datas]
+        res = 0
         if isinstance(datas, list):
             L.acquire()
             try:
@@ -688,13 +696,47 @@ class Tool:
                     kv_up = analysis_dict(data, type="update")
                     sql = insert_url.format(table, key, value, kv_up)
                     try:
-                        cursor.execute(sql)
+                        res = cursor.execute(sql)
                     except Exception as e:
                         print(e, sql)
                 self.conn.commit()
                 cursor.close()
             finally:
                 L.release()
+        return res
+
+    def save_many_data_mysql(self, table, data_list):
+        '''
+        :return:
+        '''
+        if not isinstance(data_list, list):
+            data_list = [data_list]
+        keys = data_list[0].keys()
+        sql = "insert into %s (%s) values " % (table, ",".join(keys))
+        insert_sql_list = []
+        # 对数组进行切片，每3W条为一个SQL
+        new_data_list = []
+        split_count = 30000
+        for i in range(0, len(data_list), split_count):
+            new_data_list.append(data_list[i: (i + split_count)])
+
+        for data in data_list:
+            values = []
+            for key in keys:
+                value = data.get(key, "")
+                if isinstance(value, int) or isinstance(value, float) or isinstance(value, bytes):
+                    value = '%s' % value
+                elif isinstance(value, str):
+                    value = '"%s"' % pymysql.escape_string(value)
+                elif value or value == 0:
+                    value = '"%s"' % value
+                values.append(value)
+            values = ','.join(values)
+            insert_sql = "(%s)" % values
+            insert_sql_list.append(insert_sql)
+        insert_sql_list = ",".join(insert_sql_list)
+        sql += insert_sql_list
+        self.con_execute(sql)
 
 
 if __name__ == '__main__':
